@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with gedit; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, 
+# Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA  02110-1301  USA
 
 import gedit
@@ -28,12 +28,15 @@ import gobject
 import vte
 import gconf
 import gettext
-import gnomevfs
 import os
+import gio
 from gpdefs import *
 
-gettext.bindtextdomain(GETTEXT_PACKAGE, GP_LOCALEDIR)
-_ = lambda s: gettext.dgettext("gedit-plugins", s);
+try:
+    gettext.bindtextdomain(GETTEXT_PACKAGE, GP_LOCALEDIR)
+    _ = lambda s: gettext.dgettext(GETTEXT_PACKAGE, s);
+except:
+    _ = lambda s: s
 
 class GeditTerminal(gtk.HBox):
     """VTE terminal which follows gnome-terminal default profile options"""
@@ -47,21 +50,21 @@ class GeditTerminal(gtk.HBox):
     }
 
     GCONF_PROFILE_DIR = "/apps/gnome-terminal/profiles/Default"
-    
+
     defaults = {
         'allow_bold'            : True,
         'audible_bell'          : False,
         'background'            : None,
-        'background_color'      : '#000000',
         'backspace_binding'     : 'ascii-del',
-        'cursor_blinks'         : False,
+        'cursor_blink_mode'     : vte.CURSOR_BLINK_SYSTEM,
+        'cursor_shape'          : vte.CURSOR_SHAPE_BLOCK,
         'emulation'             : 'xterm',
         'font_name'             : 'Monospace 10',
-        'foreground_color'      : '#AAAAAA',
         'scroll_on_keystroke'   : False,
         'scroll_on_output'      : False,
         'scrollback_lines'      : 100,
-        'visible_bell'          : False
+        'visible_bell'          : False,
+        'word_chars'            : '-A-Za-z0-9,./?%&#:_'
     }
 
     def __init__(self):
@@ -76,14 +79,16 @@ class GeditTerminal(gtk.HBox):
         self._vte.set_size_request(200, 50)
         self._vte.show()
         self.pack_start(self._vte)
-        
+
         self._scrollbar = gtk.VScrollbar(self._vte.get_adjustment())
         self._scrollbar.show()
         self.pack_start(self._scrollbar, False, False, 0)
-        
+
         gconf_client.notify_add(self.GCONF_PROFILE_DIR,
                                 self.on_gconf_notification)
-        
+
+        # we need to reconf colors if the style changes
+        self._vte.connect("style-set", lambda term, oldstyle: self.reconfigure_vte())
         self._vte.connect("key-press-event", self.on_vte_key_press)
         self._vte.connect("button-press-event", self.on_vte_button_press)
         self._vte.connect("popup-menu", self.on_vte_popup_menu)
@@ -91,10 +96,13 @@ class GeditTerminal(gtk.HBox):
 
         self._vte.fork_command()
 
+    def do_grab_focus(self):
+        self._vte.grab_focus()
+
     def reconfigure_vte(self):
         # Fonts
         if gconf_get_bool(self.GCONF_PROFILE_DIR + "/use_system_font"):
-            font_name = gconf_get_str("/desktop/gnome/interface/monospace_font",
+            font_name = gconf_get_str("/desktop/gnome/interface/monospace_font_name",
                                       self.defaults['font_name'])
         else:
             font_name = gconf_get_str(self.GCONF_PROFILE_DIR + "/font",
@@ -106,23 +114,61 @@ class GeditTerminal(gtk.HBox):
             pass
 
         # colors
-        fg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/foreground_color",
-                                 self.defaults['foreground_color'])
-        bg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/background_color",
-                                 self.defaults['background_color'])
-        self._vte.set_colors(gtk.gdk.color_parse (fg_color),
-                             gtk.gdk.color_parse (bg_color),
-                             [])
+        self._vte.ensure_style()
+        style = self._vte.get_style()
+        fg = style.text[gtk.STATE_NORMAL]
+        bg = style.base[gtk.STATE_NORMAL]
+        palette = []
 
-        self._vte.set_cursor_blinks(gconf_get_bool(self.GCONF_PROFILE_DIR + "/cursor_blinks",
-                                                   self.defaults['cursor_blinks']))
+        if not gconf_get_bool(self.GCONF_PROFILE_DIR + "/use_theme_colors"):
+            fg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/foreground_color", None)
+            if (fg_color):
+                fg = gtk.gdk.color_parse (fg_color)
+            bg_color = gconf_get_str(self.GCONF_PROFILE_DIR + "/background_color", None)
+            if (bg_color):
+                bg = gtk.gdk.color_parse (bg_color)
+        str_colors = gconf_get_str(self.GCONF_PROFILE_DIR + "/palette", None)
+        if (str_colors):
+            for str_color in str_colors.split(':'):
+                try:
+                    palette.append(gtk.gdk.color_parse(str_color))
+                except:
+                    palette = []
+                    break
+            if (len(palette) not in (0, 8, 16, 24)):
+                palette = []
+        self._vte.set_colors(fg, bg, palette)
+
+        # cursor blink
+        blink_mode = gconf_get_str(self.GCONF_PROFILE_DIR + "/cursor_blink_mode")
+        if blink_mode.lower() == "system":
+            blink = vte.CURSOR_BLINK_SYSTEM
+        elif blink_mode.lower() == "on":
+            blink = vte.CURSOR_BLINK_ON
+        elif blink_mode.lower() == "off":
+            blink = vte.CURSOR_BLINK_OFF
+        else:
+            blink = self.defaults['cursor_blink_mode']
+        self._vte.set_cursor_blink_mode(blink)
+
+        # cursor shape
+        cursor_shape = gconf_get_str(self.GCONF_PROFILE_DIR + "/cursor_shape")
+        if cursor_shape.lower() == "block":
+            shape = vte.CURSOR_SHAPE_BLOCK
+        elif cursor_shape.lower() == "ibeam":
+            shape = vte.CURSOR_SHAPE_IBEAM
+        elif cursor_shape.lower() == "underline":
+            shape = vte.CURSOR_SHAPE_UNDERLINE
+        else:
+            shape = self.defaults['cursor_shape']
+        self._vte.set_cursor_shape(shape)
 
         self._vte.set_audible_bell(not gconf_get_bool(self.GCONF_PROFILE_DIR + "/silent_bell",
                                                       not self.defaults['audible_bell']))
 
         self._vte.set_scrollback_lines(gconf_get_int(self.GCONF_PROFILE_DIR + "/scrollback_lines",
                                                      self.defaults['scrollback_lines']))
-        
+
         self._vte.set_allow_bold(gconf_get_bool(self.GCONF_PROFILE_DIR + "/allow_bold",
                                                 self.defaults['allow_bold']))
 
@@ -132,6 +178,9 @@ class GeditTerminal(gtk.HBox):
         self._vte.set_scroll_on_output(gconf_get_bool(self.GCONF_PROFILE_DIR + "/scroll_on_output",
                                                       self.defaults['scroll_on_output']))
 
+        self._vte.set_word_chars(gconf_get_str(self.GCONF_PROFILE_DIR + "/word_chars",
+                                               self.defaults['word_chars']))
+
         self._vte.set_emulation(self.defaults['emulation'])
         self._vte.set_visible_bell(self.defaults['visible_bell'])
 
@@ -151,6 +200,7 @@ class GeditTerminal(gtk.HBox):
 
     def on_vte_button_press(self, term, event):
         if event.button == 3:
+            self._vte.grab_focus()
             self.do_popup(event)
             return True
 
@@ -161,32 +211,41 @@ class GeditTerminal(gtk.HBox):
         menu = gtk.Menu()
 
         item = gtk.ImageMenuItem(gtk.STOCK_COPY)
-        item.connect("activate", lambda menu_item: self._vte.copy_clipboard())
+        item.connect("activate", lambda menu_item: self.copy_clipboard())
         item.set_sensitive(self._vte.get_has_selection())
         menu.append(item)
 
         item = gtk.ImageMenuItem(gtk.STOCK_PASTE)
-        item.connect("activate", lambda menu_item: self._vte.paste_clipboard())
+        item.connect("activate", lambda menu_item: self.paste_clipboard())
         menu.append(item)
-        
+
         self.emit("populate-popup", menu)
         menu.show_all()
         return menu
 
     def do_popup(self, event = None):
         menu = self.create_popup_menu()
-   
+
         if event is not None:
-	        menu.popup(None, None, None, event.button, event.time)
+            menu.popup(None, None, None, event.button, event.time)
         else:
             menu.popup(None, None,
                        lambda m: gedit.utils.menu_position_under_widget(m, self),
                        0, gtk.get_current_event_time())
-            menu.select_first(False)        
+            menu.select_first(False)
+
+    def copy_clipboard(self):
+        self._vte.copy_clipboard()
+        self._vte.grab_focus()
+
+    def paste_clipboard(self):
+        self._vte.paste_clipboard()
+        self._vte.grab_focus()
 
     def change_directory(self, path):
         path = path.replace('\\', '\\\\').replace('"', '\\"')
         self._vte.feed_child('cd "%s"\n' % path)
+        self._vte.grab_focus()
 
 class TerminalWindowHelper(object):
     def __init__(self, window):
@@ -205,7 +264,7 @@ class TerminalWindowHelper(object):
     def deactivate(self):
         bottom = self._window.get_bottom_panel()
         bottom.remove_item(self._panel)
-    
+
     def update_ui(self):
         pass
 
@@ -215,7 +274,9 @@ class TerminalWindowHelper(object):
             return None
         uri = doc.get_uri()
         if uri is not None and gedit.utils.uri_has_file_scheme(uri):
-            return os.path.dirname(gnomevfs.get_local_path_from_uri(uri))
+            gfile = gio.File(uri)
+            directory = gfile.get_parent()
+            return directory.get_path()
         return None
 
     def on_panel_populate_popup(self, panel, menu):
@@ -246,7 +307,7 @@ class TerminalPlugin(gedit.Plugin):
 gconf_client = gconf.client_get_default()
 def gconf_get_bool(key, default = False):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_BOOL:
         return val.get_bool()
     else:
@@ -254,7 +315,7 @@ def gconf_get_bool(key, default = False):
 
 def gconf_get_str(key, default = ""):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_STRING:
         return val.get_string()
     else:
@@ -262,7 +323,7 @@ def gconf_get_str(key, default = ""):
 
 def gconf_get_int(key, default = 0):
     val = gconf_client.get(key)
-    
+
     if val is not None and val.type == gconf.VALUE_INT:
         return val.get_int()
     else:
